@@ -9,12 +9,13 @@ from oasys.widgets.exchange import DataExchangeObject
 from collections import OrderedDict
 
 from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget
-from orangecontrib.xoppy.util import srundplug
+from orangecontrib.xoppy.util.xoppy_undulators import xoppy_calc_undulator_power_density
 
 from syned.widget.widget_decorator import WidgetDecorator
 import syned.beamline.beamline as synedb
 import syned.storage_ring.magnetic_structures.insertion_device as synedid
 
+
 class OWundulator_power_density(XoppyWidget, WidgetDecorator):
     name = "Undulator Power Density"
     id = "orange.widgets.dataundulator_power_density"
@@ -35,15 +36,27 @@ class OWundulator_power_density(XoppyWidget, WidgetDecorator):
     PERIODID = Setting(0.018)
     NPERIODS = Setting(222)
     KV = Setting(1.68)
+    KH = Setting(0.0)
+    KPHASE = Setting(0.0)
     DISTANCE = Setting(30.0)
-    GAPH = Setting(0.003)
-    GAPV = Setting(0.003)
+    GAPH = Setting(0.01)
+    GAPV = Setting(0.01)
     HSLITPOINTS = Setting(41)
     VSLITPOINTS = Setting(41)
-    METHOD = Setting(0)
+    METHOD = Setting(2)
+    MASK_FLAG=Setting(0)
+    MASK_ROT_H_DEG=Setting(0.0)
+    MASK_ROT_V_DEG=Setting(0.0)
+    MASK_H_MIN=Setting(-1000.0)
+    MASK_H_MAX=Setting( 1000.0)
+    MASK_V_MIN=Setting(-1000.0)
+    MASK_V_MAX=Setting( 1000.0)
 
+    H5_FILE_DUMP=Setting(0)
     inputs = WidgetDecorator.syned_input_data()
 
+    def __init__(self):
+        super().__init__(show_script_tab=True)
 
     def build_gui(self):
 
@@ -53,8 +66,6 @@ class OWundulator_power_density(XoppyWidget, WidgetDecorator):
         #
         #
         #
-
-
         idx += 1
         box1 = gui.widgetBox(box)
         gui.comboBox(box1, self, "USEEMITTANCES",
@@ -143,7 +154,24 @@ class OWundulator_power_density(XoppyWidget, WidgetDecorator):
                      label=self.unitLabels()[idx], addSpace=False,
                     valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
         self.show_at(self.unitFlags()[idx], box1) 
-        
+
+        #widget index 9 B
+        idx += 1
+        box1 = gui.widgetBox(box)
+        self.id_KH = oasysgui.lineEdit(box1, self, "KH",
+                     label=self.unitLabels()[idx], addSpace=False,
+                    valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
+        self.show_at(self.unitFlags()[idx], box1)
+
+        #widget index 9 C
+        idx += 1
+        box1 = gui.widgetBox(box)
+        self.id_KPHASE = oasysgui.lineEdit(box1, self, "KPHASE",
+                     label=self.unitLabels()[idx], addSpace=False,
+                    valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
+        self.show_at(self.unitFlags()[idx], box1)
+
+
         #widget index 10 
         idx += 1 
         box1 = gui.widgetBox(box) 
@@ -193,11 +221,96 @@ class OWundulator_power_density(XoppyWidget, WidgetDecorator):
                     valueType=int, orientation="horizontal", labelWidth=250)
         self.show_at(self.unitFlags()[idx], box1) 
 
+        #
+        # Mask
+        #
+
+        #widget index 16
+        idx += 1
+        box1 = gui.widgetBox(box)
+        gui.comboBox(box1, self, "MASK_FLAG",
+                     label=self.unitLabels()[idx], addSpace=False,
+                    items=['No', 'Yes'],
+                    valueType=int, orientation="horizontal", labelWidth=250)
+        self.show_at(self.unitFlags()[idx], box1)
+
+
+        #widget index 17
+        idx += 1
+        box1 = gui.widgetBox(box)
+        oasysgui.lineEdit(box1, self, "MASK_ROT_H_DEG",
+                     label=self.unitLabels()[idx], addSpace=False,
+                    valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
+        self.show_at(self.unitFlags()[idx], box1)
+
+        #widget index 18
+        idx += 1
+        box1 = gui.widgetBox(box)
+        oasysgui.lineEdit(box1, self, "MASK_ROT_V_DEG",
+                     label=self.unitLabels()[idx], addSpace=False,
+                    valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
+        self.show_at(self.unitFlags()[idx], box1)
+
+        #widget index 19
+        idx += 1
+        box1 = gui.widgetBox(box)
+        oasysgui.lineEdit(box1, self, "MASK_H_MIN",
+                     label=self.unitLabels()[idx], addSpace=False,
+                    valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
+        self.show_at(self.unitFlags()[idx], box1)
+
+        #widget index 20
+        idx += 1
+        box1 = gui.widgetBox(box)
+        oasysgui.lineEdit(box1, self, "MASK_H_MAX",
+                     label=self.unitLabels()[idx], addSpace=False,
+                    valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
+        self.show_at(self.unitFlags()[idx], box1)
+
+        #widget index 21
+        idx += 1
+        box1 = gui.widgetBox(box)
+        oasysgui.lineEdit(box1, self, "MASK_V_MIN",
+                     label=self.unitLabels()[idx], addSpace=False,
+                    valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
+        self.show_at(self.unitFlags()[idx], box1)
+
+        #widget index 22
+        idx += 1
+        box1 = gui.widgetBox(box)
+        oasysgui.lineEdit(box1, self, "MASK_V_MAX",
+                     label=self.unitLabels()[idx], addSpace=False,
+                    valueType=float, validator=QDoubleValidator(), orientation="horizontal", labelWidth=250)
+        self.show_at(self.unitFlags()[idx], box1)
+
+        #widget index 23
+        idx += 1
+        box1 = gui.widgetBox(box)
+        gui.comboBox(box1, self, "H5_FILE_DUMP",
+                     label=self.unitLabels()[idx], addSpace=False,
+                    items=['No', 'Yes: undulator_power_density.h5'],
+                    valueType=int, orientation="horizontal", labelWidth=250)
+        self.show_at(self.unitFlags()[idx], box1)
+
     def unitLabels(self):
-         return ["Use emittances","Electron Energy [GeV]", "Electron Energy Spread", "Electron Current [A]", "Electron Beam Size H [m]", "Electron Beam Size V [m]", "Electron Beam Divergence H [rad]", "Electron Beam Divergence V [rad]", "Period ID [m]", "Number of periods", "Kv [undulator K value vertical field]", "Distance to slit [m]", "Slit gap H [m]", "Slit gap V [m]", "Number of slit mesh points in H", "Number of slit mesh points in V", "calculation code"]
+         return ["Use emittances","Electron Energy [GeV]", "Electron Energy Spread", "Electron Current [A]",\
+                 "Electron Beam Size H [m]", "Electron Beam Size V [m]", "Electron Beam Divergence H [rad]", "Electron Beam Divergence V [rad]", \
+                 "Period ID [m]", "Number of periods", "Kv [K value vertical field]", \
+                 "Kh [K value horizontal field]","Kphase [Phase diff Kh-Kv in rad]",\
+                 "Distance to slit [m]", "Slit gap H [m]", "Slit gap V [m]", "Number of slit mesh points in H", "Number of slit mesh points in V",\
+                 "calculation code",\
+                 "modify slit","Rotation around H axis [deg]","Rotation around V axis [deg]","Mask H min [mm]","Mask H max [mm]",'Mask V min [mm]',"Mask V max [mm]",\
+                 "Dump hdf5 file"]
 
     def unitFlags(self):
-         return ["True","True", "self.USEEMITTANCES == 1 and self.METHOD != 1", "True", "self.USEEMITTANCES == 1", "self.USEEMITTANCES == 1", "self.USEEMITTANCES == 1", "self.USEEMITTANCES == 1", "True", "True", "True", "True", "True", "True", "True", "True", "True"]
+         return ["True","True", "self.USEEMITTANCES == 1 and self.METHOD != 1", "True",\
+                 "self.USEEMITTANCES == 1", "self.USEEMITTANCES == 1", "self.USEEMITTANCES == 1", "self.USEEMITTANCES == 1", \
+                 "True", "True", "True",\
+                 "self.METHOD != 0","self.METHOD != 0",\
+                 "True", "True", "True", "True", "True",\
+                 "True",\
+                 "True","self.MASK_FLAG == 1","self.MASK_FLAG == 1","self.MASK_FLAG == 1","self.MASK_FLAG == 1","self.MASK_FLAG == 1","self.MASK_FLAG == 1",\
+                 "True"]
 
     def get_help_name(self):
         return 'undulator_power_density'
@@ -214,6 +327,8 @@ class OWundulator_power_density(XoppyWidget, WidgetDecorator):
         self.PERIODID = congruence.checkStrictlyPositiveNumber(self.PERIODID, "Period ID")
         self.NPERIODS = congruence.checkStrictlyPositiveNumber(self.NPERIODS, "Number of Periods")
         self.KV = congruence.checkPositiveNumber(self.KV, "Kv")
+        self.KH = congruence.checkPositiveNumber(self.KH, "Kh")
+        self.KPHASE = congruence.checkNumber(self.KPHASE, "Kphase")
         self.DISTANCE = congruence.checkPositiveNumber(self.DISTANCE, "Distance to slit")
         self.GAPH = congruence.checkPositiveNumber(self.GAPH, "Slit gap H")
         self.GAPV = congruence.checkPositiveNumber(self.GAPV, "Slit gap V")
@@ -245,7 +360,7 @@ class OWundulator_power_density(XoppyWidget, WidgetDecorator):
 
                 try:
 
-                    print("Result arrays (shapes): ", h.shape, v.shape, p.shape )
+                    print("\nResult arrays (shapes): ", h.shape, v.shape, p.shape )
 
                     self.plot_data2D(p, h, v, 0, 0,
                                      xtitle='H [mm]',
@@ -262,7 +377,41 @@ class OWundulator_power_density(XoppyWidget, WidgetDecorator):
                 raise Exception("Empty Data")
 
     def do_xoppy_calculation(self):
-        return  xoppy_calc_undulator_power_density(ELECTRONENERGY=self.ELECTRONENERGY,
+        if self.H5_FILE_DUMP == 0:
+            h5_file = ""
+        else:
+            h5_file = "undulator_power_density.h5"
+
+        h5_parameters = {
+            "ELECTRONENERGY":self.ELECTRONENERGY,
+            "ELECTRONENERGYSPREAD":self.ELECTRONENERGYSPREAD,
+            "ELECTRONCURRENT":self.ELECTRONCURRENT,
+            "ELECTRONBEAMSIZEH":self.ELECTRONBEAMSIZEH,
+            "ELECTRONBEAMSIZEV":self.ELECTRONBEAMSIZEV,
+            "ELECTRONBEAMDIVERGENCEH":self.ELECTRONBEAMDIVERGENCEH,
+            "ELECTRONBEAMDIVERGENCEV":self.ELECTRONBEAMDIVERGENCEV,
+            "PERIODID":self.PERIODID,
+            "NPERIODS":self.NPERIODS,
+            "KV":self.KV,
+            "KH": self.KH,
+            "KPHASE": self.KPHASE,
+            "DISTANCE":self.DISTANCE,
+            "GAPH":self.GAPH,
+            "GAPV":self.GAPV,
+            "HSLITPOINTS":self.HSLITPOINTS,
+            "VSLITPOINTS":self.VSLITPOINTS,
+            "METHOD":self.METHOD,
+            "USEEMITTANCES":self.USEEMITTANCES,
+            "MASK_FLAG":self.MASK_FLAG,
+            "MASK_ROT_H_DEG":self.MASK_ROT_H_DEG,
+            "MASK_ROT_V_DEG":self.MASK_ROT_V_DEG,
+            "MASK_H_MIN":self.MASK_H_MIN,
+            "MASK_H_MAX":self.MASK_H_MAX,
+            "MASK_V_MIN":self.MASK_V_MIN,
+            "MASK_V_MAX":self.MASK_V_MAX,
+        }
+
+        h, v, p, code =  xoppy_calc_undulator_power_density(ELECTRONENERGY=self.ELECTRONENERGY,
                                                    ELECTRONENERGYSPREAD=self.ELECTRONENERGYSPREAD,
                                                    ELECTRONCURRENT=self.ELECTRONCURRENT,
                                                    ELECTRONBEAMSIZEH=self.ELECTRONBEAMSIZEH,
@@ -272,13 +421,108 @@ class OWundulator_power_density(XoppyWidget, WidgetDecorator):
                                                    PERIODID=self.PERIODID,
                                                    NPERIODS=self.NPERIODS,
                                                    KV=self.KV,
+                                                   KH=self.KH,
+                                                   KPHASE=self.KPHASE,
                                                    DISTANCE=self.DISTANCE,
                                                    GAPH=self.GAPH,
                                                    GAPV=self.GAPV,
                                                    HSLITPOINTS=self.HSLITPOINTS,
                                                    VSLITPOINTS=self.VSLITPOINTS,
                                                    METHOD=self.METHOD,
-                                                   USEEMITTANCES=self.USEEMITTANCES)
+                                                   USEEMITTANCES=self.USEEMITTANCES,
+                                                   MASK_FLAG=self.MASK_FLAG,
+                                                   MASK_ROT_H_DEG=self.MASK_ROT_H_DEG,
+                                                   MASK_ROT_V_DEG=self.MASK_ROT_V_DEG,
+                                                   MASK_H_MIN=self.MASK_H_MIN,
+                                                   MASK_H_MAX=self.MASK_H_MAX,
+                                                   MASK_V_MIN=self.MASK_V_MIN,
+                                                   MASK_V_MAX=self.MASK_V_MAX,
+                                                   h5_file=h5_file,
+                                                   h5_entry_name="XOPPY_POWERDENSITY",
+                                                   h5_initialize=True,
+                                                   h5_parameters=h5_parameters,
+                                                   )
+
+        self.xoppy_script.set_code(self.script_template().format_map(h5_parameters))
+
+        return h, v, p, code
+
+    def script_template(self):
+        return """
+#
+# script to make the calculations (created by XOPPY:undulator_spectrum)
+#
+from orangecontrib.xoppy.util.xoppy_undulators import xoppy_calc_undulator_power_density
+
+h5_parameters = dict()
+h5_parameters["ELECTRONENERGY"]=           {ELECTRONENERGY}
+h5_parameters["ELECTRONENERGYSPREAD"]=     {ELECTRONENERGYSPREAD}
+h5_parameters["ELECTRONCURRENT"]=          {ELECTRONCURRENT}
+h5_parameters["ELECTRONBEAMSIZEH"]=        {ELECTRONBEAMSIZEH}
+h5_parameters["ELECTRONBEAMSIZEV"]=        {ELECTRONBEAMSIZEV}
+h5_parameters["ELECTRONBEAMDIVERGENCEH"]=  {ELECTRONBEAMDIVERGENCEH}
+h5_parameters["ELECTRONBEAMDIVERGENCEV"]=  {ELECTRONBEAMDIVERGENCEV}
+h5_parameters["PERIODID"]=                 {PERIODID}
+h5_parameters["NPERIODS"]=                 {NPERIODS}
+h5_parameters["KV"]=                       {KV}
+h5_parameters["KH"]=                       {KH}
+h5_parameters["KPHASE"]=                   {KPHASE}
+h5_parameters["DISTANCE"]=                 {DISTANCE}
+h5_parameters["GAPH"]=                     {GAPH}
+h5_parameters["GAPV"]=                     {GAPV}
+h5_parameters["HSLITPOINTS"]=              {HSLITPOINTS}
+h5_parameters["VSLITPOINTS"]=              {VSLITPOINTS}
+h5_parameters["METHOD"]=                   {METHOD}
+h5_parameters["USEEMITTANCES"]=            {USEEMITTANCES}
+h5_parameters["MASK_FLAG"]=                {MASK_FLAG}
+h5_parameters["MASK_ROT_H_DEG"]=           {MASK_ROT_H_DEG}
+h5_parameters["MASK_ROT_V_DEG"]=           {MASK_ROT_V_DEG}
+h5_parameters["MASK_H_MIN"]=               {MASK_H_MIN}
+h5_parameters["MASK_H_MAX"]=               {MASK_H_MAX}
+h5_parameters["MASK_V_MIN"]=               {MASK_V_MIN}
+h5_parameters["MASK_V_MAX"]=               {MASK_V_MAX}
+
+        
+h, v, p, code = xoppy_calc_undulator_power_density(
+    ELECTRONENERGY           =  h5_parameters["ELECTRONENERGY"],
+    ELECTRONENERGYSPREAD     =  h5_parameters["ELECTRONENERGYSPREAD"],
+    ELECTRONCURRENT          =  h5_parameters["ELECTRONCURRENT"],
+    ELECTRONBEAMSIZEH        =  h5_parameters["ELECTRONBEAMSIZEH"],
+    ELECTRONBEAMSIZEV        =  h5_parameters["ELECTRONBEAMSIZEV"],
+    ELECTRONBEAMDIVERGENCEH  =  h5_parameters["ELECTRONBEAMDIVERGENCEH"],
+    ELECTRONBEAMDIVERGENCEV  =  h5_parameters["ELECTRONBEAMDIVERGENCEV"],
+    PERIODID                 =  h5_parameters["PERIODID"],
+    NPERIODS                 =  h5_parameters["NPERIODS"],
+    KV                       =  h5_parameters["KV"],
+    KH                       =  h5_parameters["KH"],
+    KPHASE                   =  h5_parameters["KPHASE"],
+    DISTANCE                 =  h5_parameters["DISTANCE"],
+    GAPH                     =  h5_parameters["GAPH"],
+    GAPV                     =  h5_parameters["GAPV"],
+    HSLITPOINTS              =  h5_parameters["HSLITPOINTS"],
+    VSLITPOINTS              =  h5_parameters["VSLITPOINTS"],
+    METHOD                   =  h5_parameters["METHOD"],
+    USEEMITTANCES            =  h5_parameters["USEEMITTANCES"],
+    MASK_FLAG                =  h5_parameters["MASK_FLAG"],
+    MASK_ROT_H_DEG           =  h5_parameters["MASK_ROT_H_DEG"],
+    MASK_ROT_V_DEG           =  h5_parameters["MASK_ROT_V_DEG"],
+    MASK_H_MIN               =  h5_parameters["MASK_H_MIN"],
+    MASK_H_MAX               =  h5_parameters["MASK_H_MAX"],
+    MASK_V_MIN               =  h5_parameters["MASK_V_MIN"],
+    MASK_V_MAX               =  h5_parameters["MASK_V_MAX"],
+    h5_file                  =  "undulator_power_density.h5",
+    h5_entry_name            =  "XOPPY_POWERDENSITY",
+    h5_initialize            =  True,
+    h5_parameters            =  h5_parameters,
+    )
+# example plot
+from srxraylib.plot.gol import plot_image
+plot_image(p,h,v,xtitle="H [mm]",ytitle="V [mm]",title="Power density W/mm2")
+#
+# end script
+#
+"""
+
 
     def extract_data_from_xoppy_output(self, calculation_output):
         h, v, p, code = calculation_output
@@ -315,6 +559,8 @@ class OWundulator_power_density(XoppyWidget, WidgetDecorator):
                 self.PERIODID = light_source._magnetic_structure._period_length
                 self.NPERIODS = light_source._magnetic_structure._number_of_periods
                 self.KV = light_source._magnetic_structure._K_vertical
+                self.KH = light_source._magnetic_structure._K_horizontal
+                # TODO self.KPHASE = light_source._magnetic_structure._K_vertical
 
                 self.set_enabled(False)
 
@@ -337,6 +583,8 @@ class OWundulator_power_density(XoppyWidget, WidgetDecorator):
                 self.id_PERIODID.setEnabled(True)
                 self.id_NPERIODS.setEnabled(True)
                 self.id_KV.setEnabled(True)
+                self.id_KH.setEnabled(True)
+                self.id_KPHASE.setEnabled(True)
         else:
                 self.id_ELECTRONENERGY.setEnabled(False)
                 self.id_ELECTRONENERGYSPREAD.setEnabled(False)
@@ -348,69 +596,16 @@ class OWundulator_power_density(XoppyWidget, WidgetDecorator):
                 self.id_PERIODID.setEnabled(False)
                 self.id_NPERIODS.setEnabled(False)
                 self.id_KV.setEnabled(False)
-
-
-# --------------------------------------------------------------------------------------------
-# --------------------------------------------------------------------------------------------
-
-def xoppy_calc_undulator_power_density(ELECTRONENERGY=6.04,ELECTRONENERGYSPREAD=0.001,ELECTRONCURRENT=0.2,\
-                                       ELECTRONBEAMSIZEH=0.000395,ELECTRONBEAMSIZEV=9.9e-06,\
-                                       ELECTRONBEAMDIVERGENCEH=1.05e-05,ELECTRONBEAMDIVERGENCEV=3.9e-06,\
-                                       PERIODID=0.018,NPERIODS=222,KV=1.68,DISTANCE=30.0,GAPH=0.001,GAPV=0.001,\
-                                       HSLITPOINTS=101,VSLITPOINTS=51,METHOD=0,USEEMITTANCES=1):
-    print("Inside xoppy_calc_undulator_power_density. ")
-
-    bl = OrderedDict()
-    bl['ElectronBeamDivergenceH'] = ELECTRONBEAMDIVERGENCEH
-    bl['ElectronBeamDivergenceV'] = ELECTRONBEAMDIVERGENCEV
-    bl['ElectronBeamSizeH'] = ELECTRONBEAMSIZEH
-    bl['ElectronBeamSizeV'] = ELECTRONBEAMSIZEV
-    bl['ElectronCurrent'] = ELECTRONCURRENT
-    bl['ElectronEnergy'] = ELECTRONENERGY
-    bl['ElectronEnergySpread'] = ELECTRONENERGYSPREAD
-    bl['Kv'] = KV
-    bl['NPeriods'] = NPERIODS
-    bl['PeriodID'] = PERIODID
-    bl['distance'] = DISTANCE
-    bl['gapH'] = GAPH
-    bl['gapV'] = GAPV
-
-    if USEEMITTANCES:
-        zero_emittance = False
-    else:
-        zero_emittance = True
-
-    #TODO remove SPEC file
-    outFile = "undulator_power_density.spec"
-
-    if METHOD == 0:
-        code = "US"
-        print("Undulator power_density calculation using US. Please wait...")
-        h,v,p = srundplug.calc2d_us(bl,fileName=outFile,fileAppend=False,hSlitPoints=HSLITPOINTS,vSlitPoints=VSLITPOINTS,
-                                    zero_emittance=zero_emittance)
-        print("Done")
-    if METHOD == 1:
-        code = "URGENT"
-        print("Undulator power_density calculation using URGENT. Please wait...")
-        h,v,p = srundplug.calc2d_urgent(bl,fileName=outFile,fileAppend=False,hSlitPoints=HSLITPOINTS,vSlitPoints=VSLITPOINTS,
-                                        zero_emittance=zero_emittance)
-        print("Done")
-    if METHOD == 2:
-        code = "SRW"
-        print("Undulator power_density calculation using SRW. Please wait...")
-        h,v,p = srundplug.calc2d_srw(bl,fileName=outFile,fileAppend=False,hSlitPoints=HSLITPOINTS,vSlitPoints=VSLITPOINTS,
-                                     zero_emittance=zero_emittance)
-        print("Done")
-
-    if zero_emittance:
-        print("No emittance calculation")
-
-    return h, v, p, code
+                self.id_KH.setEnabled(False)
+                self.id_KPHASE.setEnabled(False)
 
 
 if __name__ == "__main__":
+    # from srxraylib.plot.gol import plot_image
+
+
     app = QApplication(sys.argv)
     w = OWundulator_power_density()
     w.show()
     app.exec()
-    w.saveSettings()
\ No newline at end of file
+    w.saveSettings()
